@@ -4,7 +4,7 @@ package WebService::Avalara::AvaTax;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.005';     # TRIAL VERSION
+our $VERSION = '0.006';     # TRIAL VERSION
 use utf8;
 
 #pod =head1 SYNOPSIS
@@ -194,21 +194,6 @@ sub _build__transport {
 
 has _operation_name => ( is => 'rw', isa => Str, default => q{} );
 
-sub _call {
-    my ( $self, $operation, @params ) = @_;
-    $self->_operation_name($operation);
-    return $self->wsdl->call(
-        $operation => {
-            Profile => {
-                Client => "$PROGRAM_NAME," . ( $main::VERSION // q{} ),
-                Adapter => __PACKAGE__ . q{,} . ( $VERSION // q{} ),
-                Machine => hostname(),
-            },
-            parameters => {@params},
-        },
-    );
-}
-
 #pod =head1 SEE ALSO
 #pod
 #pod =over
@@ -263,21 +248,53 @@ has _stash => (
     default => sub { Package::Stash->new(__PACKAGE__) },
 );
 
+const my %OPERATION_PARAMETER => (
+    Ping         => 'Message',
+    IsAuthorized => 'Operations',
+    map { ( $_ => "${_}Request" ) }
+        qw(
+        GetTax
+        GetTaxHistory
+        PostTax
+        CommitTax
+        CancelTax
+        ReconcileTaxHistory
+        AdjustTax
+        ApplyPayment
+        TaxSummaryFetch
+        ),
+);
+
 sub _method_closure {
-    my $method = shift;
+    my $operation = shift;
     return sub {
-        my $self = shift;
+        my ( $self, @parameters ) = @_;
         my $wsdl = $self->wsdl;
-        if ( not $self->_compiled->{$method} ) {
+        if ( not $self->_compiled->{$operation} ) {
             $wsdl->compileCall(
-                $method,
+                $operation,
                 transport => $self->_transport,
                 service   => $self->_soap_service,
                 port      => $self->_soap_port,
             );
-            $self->_compiled->{$method} = 1;
+            $self->_compiled->{$operation} = 1;
         }
-        my ( $answer_ref, $trace ) = $self->_call( $method => @_ );
+
+        $self->_operation_name($operation);
+        my ( $answer_ref, $trace ) = $wsdl->call(
+            $operation => {
+                Profile => {
+                    Client => "$PROGRAM_NAME," . ( $main::VERSION // q{} ),
+                    Adapter => __PACKAGE__ . q{,} . ( $VERSION // q{} ),
+                    Machine => hostname(),
+                },
+                parameters => {
+                    $OPERATION_PARAMETER{$operation} => @parameters % 2
+                    ? "@parameters"
+                    : {@parameters},
+                },
+            },
+        );
 
         #pod =pod
         #pod
@@ -309,7 +326,7 @@ WebService::Avalara::AvaTax - Avalara SOAP interface as compiled Perl methods
 
 =head1 VERSION
 
-version 0.005
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -349,15 +366,13 @@ description of its named parameters.
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->get_tax(
-        GetTaxRequest => {
-            CustomerCode => 'ABC4335',
-            DocDate      => '2014-01-01',
-            CompanyCode  => 'APITrialCompany',
-            DocCode      => 'INV001',
-            DetailLevel  =>  'Tax',
-            Commit       => 0,
-            DocType      => 'SalesInvoice',
-        },
+        CustomerCode => 'ABC4335',
+        DocDate      => '2014-01-01',
+        CompanyCode  => 'APITrialCompany',
+        DocCode      => 'INV001',
+        DetailLevel  =>  'Tax',
+        Commit       => 0,
+        DocType      => 'SalesInvoice',
     );
 
 =head2 post_tax
@@ -365,16 +380,14 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->post_tax(
-        PostTaxRequest => {
-            CompanyCode => 'APITrialCompany',
-            DocType     => 'SalesInvoice',
-            DocCode     => 'INV001',
-            Commit      => 0,
-            DocDate     => '2014-01-01',
-            TotalTax    => '14.27',
-            TotalAmount => 175,
-            NewDocCode  => 'INV001-1',
-        },
+        CompanyCode => 'APITrialCompany',
+        DocType     => 'SalesInvoice',
+        DocCode     => 'INV001',
+        Commit      => 0,
+        DocDate     => '2014-01-01',
+        TotalTax    => '14.27',
+        TotalAmount => 175,
+        NewDocCode  => 'INV001-1',
     );
 
 =head2 commit_tax
@@ -382,12 +395,10 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->commit_tax(
-        CommitTaxRequest => {
-            DocCode     => 'INV001',
-            DocType     => 'SalesInvoice',
-            CompanyCode => 'APITrialCompany',
-            NewDocCode  => 'INV001-1',
-        },
+        DocCode     => 'INV001',
+        DocType     => 'SalesInvoice',
+        CompanyCode => 'APITrialCompany',
+        NewDocCode  => 'INV001-1',
     );
 
 =head2 cancel_tax
@@ -395,12 +406,10 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->cancel_tax(
-        CancelTaxRequest => {
-            CompanyCode => 'APITrialCompany',
-            DocType     => 'SalesInvoice',
-            DocCode     => 'INV001',
-            CancelCode  => 'DocVoided',
-        },
+        CompanyCode => 'APITrialCompany',
+        DocType     => 'SalesInvoice',
+        DocCode     => 'INV001',
+        CancelCode  => 'DocVoided',
     );
 
 =head2 adjust_tax
@@ -408,99 +417,97 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->adjust_tax(
-        AdjustTaxRequest => {
-            AdjustmentReason      => 4,
-            AdjustmentDescription => 'Transaction Adjusted for Testing',
-            GetTaxRequest => {
-                CustomerCode => 'ABC4335',
-                DocDate      => '2014-01-01',
-                CompanyCode  => 'APITrialCompany',
-                DocCode      => 'INV001',
-                DetailLevel  => 'Tax',
-                Commit       => 0,
-                DocType      => 'SalesInvoice',
-                # BusinessIdentificationNo => '234243',
-                # CustomerUsageType        => 'G',
-                # ExemptionNo              => '12345',
-                # Discount                 => 50,
-                # LocationCode             => '01',
-                # TaxOverride => [
-                #    {   TaxOverrideType => 'TaxDate',
-                #        Reason          => 'Adjustment for return',
-                #        TaxDate         => '2013-07-01',
-                #        TaxAmount       => 0,
-                #    },
-                # ],
-                # ServiceMode => 'Automatic',
-                PurchaseOrderNo     => 'PO123456',
-                ReferenceCode       => 'ref123456',
-                PosLaneCode         => '09',
-                CurrencyCode        => 'USD',
-                ExchangeRate        => '1.0',
-                ExchangeRateEffDate => '2013-01-01',
-                SalespersonCode     => 'Bill Sales',
-                Addresses => {BaseAddress => [
-                    {   AddressCode => '01',
-                        Line1       => '45 Fremont Street',
-                        City        => 'San Francisco',
-                        Region      => 'CA',
-                    },
-                    {   AddressCode => '02',
-                        Line1       => '118 N Clark St',
-                        Line2       => 'Suite 100',
-                        Line3       => 'ATTN Accounts Payable',
-                        City        => 'Chicago',
-                        Region      => 'IL',
-                        Country     => 'US',
-                        PostalCode  => '60602',
-                    },
-                    {   AddressCode => '03',
-                        Latitude    => '47.627935',
-                        Longitude   => '-122.51702',
-                    },
-                ] },
-                Lines => {Line => [
-                    {   No              => '01',
-                        ItemCode        => 'N543',
-                        Qty             => 1,
-                        Amount          => 10,
-                        TaxCode         => 'NT',
-                        Description     => 'Red Size 7 Widget',
-                        OriginCode      => '01',
-                        DestinationCode => '02',
-                        # CustomerUsageType => 'L',
-                        # ExemptionNo       => '12345',
-                        # Discounted        => 1,
-                        # TaxIncluded       => 1,
-                        # TaxOverride => {
-                        #     TaxOverrideType => 'TaxDate',
-                        #     Reason          => 'Adjustment for return',
-                        #     TaxDate         => '2013-07-01',
-                        #     TaxAmount       => 0,
-                        # },
-                        Ref1 => 'ref123',
-                        Ref2 => 'ref456',
-                    },
-                    {   No              => '02',
-                        ItemCode        => 'T345',
-                        Qty             => 3,
-                        Amount          => 150,
-                        OriginCode      => '01',
-                        DestinationCode => '03',
-                        Description     => 'Size 10 Green Running Shoe',
-                        TaxCode         => 'PC30147',
-                    },
-                    {   No              => '02-FR',
-                        ItemCode        => 'FREIGHT',
-                        Qty             => 1,
-                        Amount          => 15,
-                        OriginCode      => '01',
-                        DestinationCode => '03',
-                        Description     => 'Shipping Charge',
-                        TaxCode         => 'FR',
-                    },
-                ] },
-            },
+        AdjustmentReason      => 4,
+        AdjustmentDescription => 'Transaction Adjusted for Testing',
+        GetTaxRequest => {
+            CustomerCode => 'ABC4335',
+            DocDate      => '2014-01-01',
+            CompanyCode  => 'APITrialCompany',
+            DocCode      => 'INV001',
+            DetailLevel  => 'Tax',
+            Commit       => 0,
+            DocType      => 'SalesInvoice',
+            # BusinessIdentificationNo => '234243',
+            # CustomerUsageType        => 'G',
+            # ExemptionNo              => '12345',
+            # Discount                 => 50,
+            # LocationCode             => '01',
+            # TaxOverride => [
+            #    {   TaxOverrideType => 'TaxDate',
+            #        Reason          => 'Adjustment for return',
+            #        TaxDate         => '2013-07-01',
+            #        TaxAmount       => 0,
+            #    },
+            # ],
+            # ServiceMode => 'Automatic',
+            PurchaseOrderNo     => 'PO123456',
+            ReferenceCode       => 'ref123456',
+            PosLaneCode         => '09',
+            CurrencyCode        => 'USD',
+            ExchangeRate        => '1.0',
+            ExchangeRateEffDate => '2013-01-01',
+            SalespersonCode     => 'Bill Sales',
+            Addresses => { BaseAddress => [
+                {   AddressCode => '01',
+                    Line1       => '45 Fremont Street',
+                    City        => 'San Francisco',
+                    Region      => 'CA',
+                },
+                {   AddressCode => '02',
+                    Line1       => '118 N Clark St',
+                    Line2       => 'Suite 100',
+                    Line3       => 'ATTN Accounts Payable',
+                    City        => 'Chicago',
+                    Region      => 'IL',
+                    Country     => 'US',
+                    PostalCode  => '60602',
+                },
+                {   AddressCode => '03',
+                    Latitude    => '47.627935',
+                    Longitude   => '-122.51702',
+                },
+            ] },
+            Lines => { Line => [
+                {   No              => '01',
+                    ItemCode        => 'N543',
+                    Qty             => 1,
+                    Amount          => 10,
+                    TaxCode         => 'NT',
+                    Description     => 'Red Size 7 Widget',
+                    OriginCode      => '01',
+                    DestinationCode => '02',
+                    # CustomerUsageType => 'L',
+                    # ExemptionNo       => '12345',
+                    # Discounted        => 1,
+                    # TaxIncluded       => 1,
+                    # TaxOverride => {
+                    #     TaxOverrideType => 'TaxDate',
+                    #     Reason          => 'Adjustment for return',
+                    #     TaxDate         => '2013-07-01',
+                    #     TaxAmount       => 0,
+                    # },
+                    Ref1 => 'ref123',
+                    Ref2 => 'ref456',
+                },
+                {   No              => '02',
+                    ItemCode        => 'T345',
+                    Qty             => 3,
+                    Amount          => 150,
+                    OriginCode      => '01',
+                    DestinationCode => '03',
+                    Description     => 'Size 10 Green Running Shoe',
+                    TaxCode         => 'PC30147',
+                },
+                {   No              => '02-FR',
+                    ItemCode        => 'FREIGHT',
+                    Qty             => 1,
+                    Amount          => 15,
+                    OriginCode      => '01',
+                    DestinationCode => '03',
+                    Description     => 'Shipping Charge',
+                    TaxCode         => 'FR',
+                },
+            ] },
         },
     );
 
@@ -509,30 +516,26 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->get_tax_history(
-        GetTaxHistoryRequest => {
-            CompanyCode => 'APITrialCompany',
-            DocType     => 'SalesInvoice',
-            DocCode     => 'INV001',
-            DetailLevel => 'Tax',
-        },
+        CompanyCode => 'APITrialCompany',
+        DocType     => 'SalesInvoice',
+        DocCode     => 'INV001',
+        DetailLevel => 'Tax',
     );
 
 =head2 is_authorized
 
 Example:
 
-    my ( $answer_ref, $trace ) = $avatax->is_authorized(
-        Operations => join ', ' => qw(
-            Ping
-            IsAuthorized
-            GetTax
-            PostTax
-            GetTaxHistory
-            CommitTax
-            CancelTax
-            AdjustTax
-        ),
-    );
+    my ( $answer_ref, $trace ) = $avatax->is_authorized( join ', ' => qw(
+        Ping
+        IsAuthorized
+        GetTax
+        PostTax
+        GetTaxHistory
+        CommitTax
+        CancelTax
+        AdjustTax
+    ) );
 
 =head2 ping
 
@@ -556,17 +559,15 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->reconcile_tax_history(
-        ReconcileTaxHistoryRequest => {
-            CompanyCode => 'APITrialCompany',
-            LastDocId   => 'example',
-            Reconciled  => 1,
-            StartDate   => '2014-01-01',
-            EndDate     => '2014-01-31',
-            DocStatus   => 'Temporary',
-            DocType     => 'SalesOrder',
-            LastDocCode => 'example',
-            PageSize    => 10,
-        },
+        CompanyCode => 'APITrialCompany',
+        LastDocId   => 'example',
+        Reconciled  => 1,
+        StartDate   => '2014-01-01',
+        EndDate     => '2014-01-31',
+        DocStatus   => 'Temporary',
+        DocType     => 'SalesOrder',
+        LastDocCode => 'example',
+        PageSize    => 10,
     );
 
 =head2 apply_payment
@@ -574,13 +575,11 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->apply_payment(
-        ApplyPaymentRequest => {
-            DocId       => 'example',
-            CompanyCode => 'APITrialCompany',
-            DocType     => 'SalesInvoice',
-            DocCode     => 'INV001',
-            PaymentDate => '2014-01-01',
-        },
+        DocId       => 'example',
+        CompanyCode => 'APITrialCompany',
+        DocType     => 'SalesInvoice',
+        DocCode     => 'INV001',
+        PaymentDate => '2014-01-01',
     );
 
 =head2 tax_summary_fetch
@@ -588,11 +587,9 @@ Example:
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->tax_summary_fetch(
-        TaxSummaryFetchRequest => {
-            MerchantCode => 'example',
-            StartDate    => '2014-01-01',
-            EndDate      => '2014-01-31',
-        },
+        MerchantCode => 'example',
+        StartDate    => '2014-01-01',
+        EndDate      => '2014-01-31',
     );
 
 =head1 ATTRIBUTES
